@@ -55,7 +55,7 @@ class Lepton(Core):
             path = "/sys/class/video4linux/"
             video_devices = [os.path.join(path, device) for device in os.listdir(path)]
             dev = []
-            for i, device in enumerate(video_devices):
+            for device in video_devices:
                 udev = pyudev.Devices.from_path(context, device)
 
                 try:
@@ -63,7 +63,7 @@ class Lepton(Core):
                     pid = udev.properties['ID_MODEL_ID']
 
                     if vid.lower() == "1e4e" and pid.lower() == "0100":
-                        dev.append(i)
+                        dev.append(int(device.split('video')[-1]))
                 except KeyError:
                     pass
             
